#!/bin/bash -p
#
# This executable encapsulates all the code which be must run as root once the
# application has been copied to a new location. It is written as a script
# so invokers can more easily see what it does. It is designed to be invoked in
# two ways:
#
# 1) Interactively, by a home user running the application.
#
# When the application starts, it invokes this executable as the current user
# with the first argument set to "test", to determine whether something needs
# to be done as root (non-zero exit status) or not (zero exit status). In case
# something needs to be done, the application prompts the user for root
# credentials, then invokes this executable again as root with the first
# argument set to "set", to actually do what needs to be done as root.
#
# 2) Unattended, by the system administrator of an organization who mass
#    deploys the application.
#
# After the application has been copied to a new machine, the system
# administrator invokes this executable as root with the first argument set to
# "set". After that, any user of the organization who starts the application on
# this machine will not be prompted for root credentials.
#


# Print a variable, even if its value is -n (unlike echo).
print_var() {
   printf -- %s "$1"
}


# Canonicalize the path of an existing directory.
canonicalize_dir() {
   # Bash's cd builtin can write to stdout.
   (cd -- "$1" >/dev/null && pwd -P)
}


# Canonicalize the path of an existing file.
canonicalize_file() {
   local file="$1"

   while [ -h "$file" ]; do
      file="`readlink -- "$file"`"
   done

   local dir
   # Keep the "local" declaration separate from command substitution. PR 1882121.
   dir="`dirname -- "$file"`"

   # 'dir' is a directory, or a symlink to a directory. Canonicalize it.
   local canonicalFile
   # Keep the "local" declaration separate from command substitution. PR 1882121.
   canonicalFile="`canonicalize_dir "$dir"`"
   # Keep the above and below command substitutions separate. PR 1882121.
   canonicalFile="$canonicalFile"/"`basename -- "$file"`"
   # Keep the above and below command substitutions separate. PR 1882121.
   print_var "$canonicalFile"
}


# Test if this executable can do things as root.
canSet() {
   [ "$action" = set -a "$EUID" = 0 ]
}


# Convert a numerical access permission into a string.
accessNum2Str() {
   local result=''

   if [ "$((0$1 & 0400))" != 0 ]; then
      result="$result"r
   else
      result="$result"-
   fi
   if [ "$((0$1 & 0200))" != 0 ]; then
      result="$result"w
   else
      result="$result"-
   fi
   if [ "$((0$1 & 0100))" != 0 ]; then
      if [ "$((0$1 & 04000))" != 0 ]; then
         result="$result"s
      else
         result="$result"x
      fi
   else
      result="$result"-
   fi

   if [ "$((0$1 & 0040))" != 0 ]; then
      result="$result"r
   else
      result="$result"-
   fi
   if [ "$((0$1 & 0020))" != 0 ]; then
      result="$result"w
   else
      result="$result"-
   fi
   if [ "$((0$1 & 0010))" != 0 ]; then
      result="$result"x
   else
      result="$result"-
   fi

   if [ "$((0$1 & 0004))" != 0 ]; then
      result="$result"r
   else
      result="$result"-
   fi
   if [ "$((0$1 & 0002))" != 0 ]; then
      result="$result"w
   else
      result="$result"-
   fi
   if [ "$((0$1 & 0001))" != 0 ]; then
      result="$result"x
   else
      result="$result"-
   fi

   print_var "$result"
}


# Test the Type, Permission and Ownership of a path.
hasTPO() {
   local path="$1"
   local accessStr="$2"
   local uid="$3"
   local gid="$4"

   # Numeric IDs are used because some systems have two names (or more)
   # associated with one number.
   set -- `ls -alnd -- "$path" 2>/dev/null`
   [ "${1:0:10}" = "$accessStr" -a "$3" = "$uid" -a "$4" = "$gid" ]
}


# Test and set existence, permission and ownership of a directory.
checkDir() {
   local path="$1"
   local accessNum="$2"
   local gid="$3"
   local gName="$4"

   if ! hasTPO "$path" d"`accessNum2Str "$accessNum"`" 0 "$gid"; then
      canSet

      if ! [ -d "$path" ]; then
         echo '*** Creating directory '"$path"'...'
         mkdir -- "$path"
      fi

      echo '*** Setting permissions on '"$path"'...'
      chown -- root:"$gName" "$path"
      chmod -- "$accessNum" "$path"
   fi
}


# Verify dir is not empty nor root dir. PR 1882121
verifyDir() {
   [ "$1" != "/" -a "$1" != "" ]
}


# Set the location variables.
locationInit() {
   # PR 1882121
   # set -e doesn't propagate into a command substitution environment.
   # Add set -e in command substitution environment.
   this="`set -e && canonicalize_file "$1"`"
   libDir="`dirname -- "$this"`"
   local contentsDir
   # Keep the "local" declaration separate from command substitution. PR 1882121.
   contentsDir="`dirname -- "$libDir"`"
   appDir="`dirname -- "$contentsDir"`"
}


#
# Main
#

# To debug, uncomment the next 2 lines.
#set -x
#exec >>/tmp/debug 2>&1

set -e
# This executable runs with root privileges, so hardcode a PATH where
# unprivileged users cannot write.
export PATH=/bin:/sbin:/usr/bin:/usr/sbin

# Force the rest of this executable to explicitly set all file permissions.
umask 777

action="$1"
guiPid="$2"
newAppDir="$3"
licenseKey="$4"

if [ "$guiPid" != '' ]; then
   # When launched from GUI, Send stderr to stdout and be verbose.
   exec 2>&1
   set -x
fi

locationInit "$0"

if canSet && [ "$newAppDir" != '' ]; then
   rm -rf -- "$newAppDir"
   cp -pR -- "$appDir" "$newAppDir"
   locationInit "$newAppDir"'/Contents/Library/Initialize VMRC.tool'
fi

# Validating appDir.
verifyDir "$appDir"

SUPPORT_DIR='/Library/Application Support/VMware'
checkDir "$SUPPORT_DIR" 755 0 wheel
checkDir "$SUPPORT_DIR"'/VMware Remote Console' 755 0 wheel

# Before you change gName/gid/access below, read comment 11 of bug 1085325.
gName=wheel
gid=0
access=644
xAccess="`printf -- %o "$((0$access | 0111))"`"
sxAccess="`printf -- %o "$((0$xAccess | 04000))"`"
sxAccessStr=-"`accessNum2Str "$sxAccess"`"
# Test if a path is a set-UID root file.
isSetuidRoot() {
   hasTPO "$1" "$sxAccessStr" 0 "$gid"
}
setPerms() {
   local path

   echo '*** Setting file permissions...'

   # First, the default rule.
   # Building the product as an unprivileged user requires write permission to
   # all directories inside the build tree, to be able to update files.
   if ! [ -e "$appDir"'/Contents/Resources/Inside Build Tree' ]; then
      # All files are owned by root:"$gName".
      chown -R -- root:"$gName" "$appDir"
   fi

   # Then mark some files as executable.
   # The substitution of MAIN_EXECS uses "$appDir".
   execs=("$appDir""/Contents/MacOS/VMware Remote Console" "$appDir""/Contents/Library/VMware Remote Console Problem Reporter.tool" "$appDir""/Contents/Library/vmware-usbarbitrator" "$appDir""/Contents/Library/services/Open VMware USB Arbitrator Service" "$appDir""/Contents/Library/services/VMware USB Arbitrator Service" "$appDir""/Contents/Library/services/usbarbService.sh" "$appDir""/Contents/Library/services/VMRC Services" "$appDir""/Contents/Library/services/services.sh" "$appDir""/Contents/Library/vmware-aewp" "$appDir""/Contents/Library/vmware-id" "$appDir""/Contents/Library/Initialize VMRC.tool" )
   echo chmod -- "$xAccess" "${execs[@]}"

   # Finally mark some files as set-UID root executables.
   for path in "$@"; do
      # chown/chmod commands may return 0 even on failure (for example
      # on encrypted volumes which are mounted by default with options
      # "nosuid, noowners"). So check again after invoking them.
      chown -- root:"$gName" "$path"
      chmod -- "$sxAccess" "$path"
      isSetuidRoot "$path"
   done
}
perms() {
   local path

   for path in "$@"; do
      if ! isSetuidRoot "$path"; then
         canSet

         setPerms "$@"
         break
      fi
   done
}
# The substitution of MAIN_SETUID_EXECS uses "$appDir".
setuidExecs=("$appDir""/Contents/Library/services/Open VMRC Services" )
perms "${setuidExecs[@]}"

if canSet; then
   if [ "$guiPid" != '' ]; then
      # Use -9 so Mac OS does not popup a "VMware Fusion quit unexpectedly."
      # message.
      kill -9 -- "$guiPid"
      # Signal delivery is asynchronous. Wait until the process has effectively
      # died.
      while kill -0 -- "$guiPid" 2>/dev/null; do
         true
      done

      open -- "$appDir"
   fi
fi
